@@ -28,6 +28,9 @@
  *
  * \author Ian Romanick <idr@us.ibm.com>
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <stdlib.h>
 #include <string.h>
@@ -36,7 +39,7 @@
 #include "pciaccess.h"
 #include "pciaccess_private.h"
 
-#if defined(__linux__) || defined(__GLIBC__)
+#if defined(__linux__) || defined(__GLIBC__) || defined(__CYGWIN__)
 #include <byteswap.h>
 
 #if __BYTE_ORDER == __BIG_ENDIAN
@@ -67,6 +70,24 @@
 # define HTOLE_32(x)   (x)
 #endif /* Solaris */
 
+#elif defined(__APPLE__)
+#include <libkern/OSByteOrder.h>
+
+#define htobe16(x) OSSwapHostToBigInt16(x)
+#define htole16(x) OSSwapHostToLittleInt16(x)
+#define be16toh(x) OSSwapBigToHostInt16(x)
+#define le16toh(x) OSSwapLittleToHostInt16(x)
+
+#define htobe32(x) OSSwapHostToBigInt32(x)
+#define htole32(x) OSSwapHostToLittleInt32(x)
+#define be32toh(x) OSSwapBigToHostInt32(x)
+#define le32toh(x) OSSwapLittleToHostInt32(x)
+
+#define htobe64(x) OSSwapHostToBigInt64(x)
+#define htole64(x) OSSwapHostToLittleInt64(x)
+#define be64toh(x) OSSwapBigToHostInt64(x)
+#define le64toh(x) OSSwapLittleToHostInt64(x)
+
 #else
 
 #include <sys/endian.h>
@@ -86,14 +107,14 @@
 
 /**
  * Read a device's expansion ROM.
- * 
+ *
  * Reads the device's expansion ROM and stores the data in the memory pointed
  * to by \c buffer.  The buffer must be at least \c pci_device::rom_size
  * bytes.
  *
  * \param dev    Device whose expansion ROM is to be read.
  * \param buffer Memory in which to store the ROM.
- * 
+ *
  * \return
  * Zero on success or an \c errno value on failure.
  */
@@ -125,7 +146,7 @@ pci_device_is_boot_vga( struct pci_device * dev )
 
 /**
  * Probe a PCI device to determine if a kernel driver is attached.
- * 
+ *
  * \param dev Device to query
  * \return
  * Zero if no driver attached, 1 if attached kernel drviver
@@ -140,14 +161,14 @@ pci_device_has_kernel_driver( struct pci_device * dev )
 
 /**
  * Probe a PCI device to learn information about the device.
- * 
+ *
  * Probes a PCI device to learn various information about the device.  Before
  * calling this function, the only public fields in the \c pci_device
  * structure that have valid values are \c pci_device::domain,
  * \c pci_device::bus, \c pci_device::dev, and \c pci_device::func.
- * 
+ *
  * \param dev  Device to be probed.
- * 
+ *
  * \return
  * Zero on success or an \c errno value on failure.
  */
@@ -173,7 +194,7 @@ pci_device_probe( struct pci_device * dev )
  * \param dev          Device whose memory region is to be mapped.
  * \param region       Region, on the range [0, 5], that is to be mapped.
  * \param write_enable Map for writing (non-zero).
- * 
+ *
  * \return
  * Zero on success or an \c errno value on failure.
  *
@@ -194,7 +215,7 @@ pci_device_map_region(struct pci_device * dev, unsigned region,
     if (dev->regions[region].memory != NULL) {
         return 0;
     }
-    
+
     return pci_device_map_range(dev, dev->regions[region].base_addr,
                                 dev->regions[region].size, map_flags,
                                 &dev->regions[region].memory);
@@ -213,7 +234,7 @@ pci_device_map_region(struct pci_device * dev, unsigned region,
  * \param size         Size of the range to be mapped.
  * \param write_enable Map for writing (non-zero).
  * \param addr         Location to store the mapped address.
- * 
+ *
  * \return
  * Zero on success or an \c errno value on failure.
  *
@@ -241,7 +262,7 @@ int pci_device_map_memory_range(struct pci_device *dev,
  * \param size         Size of the range to be mapped.
  * \param map_flags    Flag bits controlling how the mapping is accessed.
  * \param addr         Location to store the mapped address.
- * 
+ *
  * \return
  * Zero on success or an \c errno value on failure.
  *
@@ -268,7 +289,7 @@ pci_device_map_range(struct pci_device *dev, pciaddr_t base,
 
 
     for (region = 0; region < 6; region++) {
-        const struct pci_mem_region const* r = &dev->regions[region];
+        const struct pci_mem_region * const r = &dev->regions[region];
 
         if (r->size != 0) {
             if ((r->base_addr <= base) && ((r->base_addr + r->size) > base)) {
@@ -335,7 +356,7 @@ pci_device_map_range(struct pci_device *dev, pciaddr_t base,
  *
  * \param dev          Device whose memory region is to be mapped.
  * \param region       Region, on the range [0, 5], that is to be mapped.
- * 
+ *
  * \return
  * Zero on success or an \c errno value on failure.
  *
@@ -374,7 +395,7 @@ pci_device_unmap_region( struct pci_device * dev, unsigned region )
  * \param dev          Device whose memory is to be unmapped.
  * \param memory       Pointer to the base of the mapped range.
  * \param size         Size, in bytes, of the range to be unmapped.
- * 
+ *
  * \return
  * Zero on success or an \c errno value on failure.
  *
@@ -398,7 +419,7 @@ pci_device_unmap_memory_range(struct pci_device *dev, void *memory,
  * \param dev          Device whose memory is to be unmapped.
  * \param memory       Pointer to the base of the mapped range.
  * \param size         Size, in bytes, of the range to be unmapped.
- * 
+ *
  * \return
  * Zero on success or an \c errno value on failure.
  *
@@ -429,17 +450,17 @@ pci_device_unmap_range(struct pci_device *dev, void *memory,
         return ENOENT;
     }
 
-    
+
     err = (*pci_sys->methods->unmap_range)(dev, &devp->mappings[i]);
     if (!err) {
         const unsigned entries_to_move = (devp->num_mappings - i) - 1;
-        
+
         if (entries_to_move > 0) {
             (void) memmove(&devp->mappings[i],
                            &devp->mappings[i + 1],
                            entries_to_move * sizeof(devp->mappings[0]));
         }
-        
+
         devp->num_mappings--;
         devp->mappings = realloc(devp->mappings,
                                  (sizeof(devp->mappings[0]) * devp->num_mappings));
@@ -474,7 +495,7 @@ pci_device_unmap_range(struct pci_device *dev, void *memory,
  */
 int
 pci_device_cfg_read( struct pci_device * dev, void * data,
-		     pciaddr_t offset, pciaddr_t size, 
+		     pciaddr_t offset, pciaddr_t size,
 		     pciaddr_t * bytes_read )
 {
     pciaddr_t  scratch;
@@ -484,7 +505,7 @@ pci_device_cfg_read( struct pci_device * dev, void * data,
     }
 
     return pci_sys->methods->read( dev, data, offset, size,
-				   (bytes_read == NULL) 
+				   (bytes_read == NULL)
 				   ? & scratch : bytes_read );
 }
 
@@ -495,7 +516,7 @@ pci_device_cfg_read_u8( struct pci_device * dev, uint8_t * data,
 {
     pciaddr_t bytes;
     int err = pci_device_cfg_read( dev, data, offset, 1, & bytes );
-    
+
     if ( (err == 0) && (bytes != 1) ) {
 	err = ENXIO;
     }
@@ -510,7 +531,7 @@ pci_device_cfg_read_u16( struct pci_device * dev, uint16_t * data,
 {
     pciaddr_t bytes;
     int err = pci_device_cfg_read( dev, data, offset, 2, & bytes );
-    
+
     if ( (err == 0) && (bytes != 2) ) {
 	err = ENXIO;
     }
@@ -561,7 +582,7 @@ pci_device_cfg_read_u32( struct pci_device * dev, uint32_t * data,
  */
 int
 pci_device_cfg_write( struct pci_device * dev, const void * data,
-		      pciaddr_t offset, pciaddr_t size, 
+		      pciaddr_t offset, pciaddr_t size,
 		      pciaddr_t * bytes_written )
 {
     pciaddr_t  scratch;
@@ -571,7 +592,7 @@ pci_device_cfg_write( struct pci_device * dev, const void * data,
     }
 
     return pci_sys->methods->write( dev, data, offset, size,
-				    (bytes_written == NULL) 
+				    (bytes_written == NULL)
 				    ? & scratch : bytes_written );
 }
 
@@ -590,7 +611,7 @@ pci_device_cfg_write_u8(struct pci_device *dev, uint8_t data,
 
     return err;
 }
-  
+
 
 int
 pci_device_cfg_write_u16(struct pci_device *dev, uint16_t data,
@@ -627,7 +648,7 @@ pci_device_cfg_write_u32(struct pci_device *dev, uint32_t data,
 
 
 int
-pci_device_cfg_write_bits( struct pci_device * dev, uint32_t mask, 
+pci_device_cfg_write_bits( struct pci_device * dev, uint32_t mask,
 			   uint32_t data, pciaddr_t offset )
 {
     uint32_t  temp;
@@ -654,3 +675,57 @@ pci_device_enable(struct pci_device *dev)
     if (pci_sys->methods->enable)
 	pci_sys->methods->enable(dev);
 }
+
+void
+pci_device_disable(struct pci_device *dev)
+{
+	if (dev == NULL)
+		return;
+
+	if (pci_sys->methods->disable)
+		pci_sys->methods->disable(dev);
+}
+
+/**
+ * Map the legacy memory space for the PCI domain containing \c dev.
+ *
+ * \param dev          Device whose memory region is to be mapped.
+ * \param base         Base address of the range to be mapped.
+ * \param size         Size of the range to be mapped.
+ * \param map_flags    Flag bits controlling how the mapping is accessed.
+ * \param addr         Location to store the mapped address.
+ *
+ * \returns
+ * Zero on success or an \c errno value on failure.
+ */
+int
+pci_device_map_legacy(struct pci_device *dev, pciaddr_t base, pciaddr_t size,
+		      unsigned map_flags, void **addr)
+{
+    if (base > 0x100000 || base + size > 0x100000)
+	return EINVAL;
+
+    if (!pci_sys->methods->map_legacy)
+	return ENOSYS;
+
+    return pci_sys->methods->map_legacy(dev, base, size, map_flags, addr);
+}
+
+/**
+ * Unmap the legacy memory space for the PCI domain containing \c dev.
+ *
+ * \param dev          Device whose memory region is to be unmapped.
+ * \param addr         Location of the mapped address.
+ * \param size         Size of the range to be unmapped.
+ *
+ * \returns
+ * Zero on success or an \c errno value on failure.
+ */
+int
+pci_device_unmap_legacy(struct pci_device *dev, void *addr, pciaddr_t size)
+{
+    if (!pci_sys->methods->unmap_legacy)
+	return ENOSYS;
+
+    return pci_sys->methods->unmap_legacy(dev, addr, size);
+}
